<?php # $Id: incl_getgps.php 209 2014-12-30 23:16:54Z zw $

# General options
require 'incl_all.php';

# Full path to ffmeg
define('ffmpeg', '/usr/bin/ffmpeg');

# Options
define('Distance', 2500);
define('MinDistance', 15);
define('Turn', 5);
define('Curve', 50);
define('MinCurve', 5);
define('Speed', .25);
define('Geo2M', 1e6/9);

# Decode latitude/longitude from the GPRMC format and store as decimal degrees
# required by the Google Maps API
function gpsdecode($val, $sign) {
$val = "00$val";
if (preg_match('/^(\d*)(\d\d\.\d*)/', $val, $m)) $val = 0 + $m[1] + $m[2] / 60;
else $val = 0;
if ($sign == 'S' || $sign == 'W') $val = -$val;
return $val;
}

# Calculate turning angle (change of heading)
function turn($angle1, $angle2) {
$turn = abs($angle1 - $angle2);
while ($turn > 180) $turn -= 180;
return $turn;
}

# Initialization
$image = 'intro.jpg';
$path = array();
$invalid = array();
$lastlng = $lastlat = $seclat - $seclng = $lastangle = $sectionangle = $lastspeed = 1234; # nonsense value
$idx = $major = $minor = -1;
$playlist = array();
$offset = array();
$tot = 0;

# Run through all files =======================================================================
foreach ($_POST['files'] as $file) {

# Get the file name from the POST data
$vfile = VideosRoot . $_POST['dir'] . '/' . $file;

# We have to find the duration and the stream id of the subtitles
exec(ffmpeg . " -i $vfile 2>&1", $output);
$duration = -1;
$substream = '';
foreach ($output as $outline) {
  if (preg_match('/Duration:\s*(\d+):(\d+):(\d*(\.\d*)?)/', $outline, $m)) {
    $duration = (60 * $m[1] + $m[2]) * 60 + $m[3];
  }
  if (preg_match('/Stream\s+#\s*(\d+:\d+).*\sSubtitle:/', $outline, $m)) $substream = $m[1];
}

# Ignore the file if duration is nonpositive
if ($duration <= 0) continue;

# Update the total duration and add it to the offsets
$offset[] = $off = $tot;
$tot += $duration;

# Add the file to the playlist
$playlist[] = array (
  'image' => $image,
  'file' => $vfile,
  'title' => $file);
$vindex = count($playlist) - 1;

# Extract the subtitles to a file or use an existing file
$subfn = preg_replace('/^(.+)\.[^.]+$/', '$1.srt', $vfile);
if (file_exists($subfn)) $tmpfn = '';
else {
  $tmpfn = tempnam(TMPDIR, 'gps'); # the file is always created
  $subfn = "$tmpfn.srt"; # ffmpeg neads the extension
  if ($duration > 0 && $substream) {
    exec(ffmpeg . " -i $vfile -vn -an -codec:s:$substream srt $subfn 2>/dev/null");
  }
}

# Examining subtitles, creating the path
$sub = fopen($subfn, 'r');
while ($sub && ($line = fgets($sub)) !== false) {
  if (preg_match('/^(\d+):(\d+):(\d+),(\d+)\s*-->/', $line, $m)) {# subtitle with no GPS data
    $s = 0 + "$m[3].$m[4]";
    $subtime = $off + (60 * $m[1] + $m[2]) * 60 + $s;
  } elseif (preg_match('/GPRMC,(.+)\*/', $line, $m)) {# GPS data, I cannot decode G-sensor data
    list($gmt, $flag, $latEnc, $latSign, $lngEnc, $lngSign, $speed, $angle, $time, $rest)
          = explode(',', $m[1]);
    if ($flag == 'A') {# Valid GPS data
      $lat = gpsdecode($latEnc, $latSign);
      $lng = gpsdecode($lngEnc, $lngSign);
      if ($speed) $speed *= 1.852; else $speed = -1; # convert to km/h
      $angle = 0 + $angle;
      $turn = turn($lastangle, $angle);
      $sturn = turn($sectionangle, $angle);
      $distance = sqrt(pow($lat-$lastlat, 2) + pow($lng-$lastlng, 2)) * Geo2M;
      $Distance = sqrt(pow($lat-$seclat, 2) + pow($lng-$seclng, 2)) * Geo2M;
      if ($Distance > Distance || ($turn > Turn && $distance > MinCurve) || ($sturn > Turn && $Distance > Curve)) {
        $prev = $minor + 1;
        $major++;
        $minor = -1;
        $sectionangle = $angle;
        $seclat = $lat;
        $seclng = $lng;
      } elseif ($distance <= MinDistance && abs($speed - $lastspeed) < Speed) continue; # ignore very short elements
      $lastlng = $lng;
      $lastlat = $lat;
      $lastangle = $angle;
      $lastspeed = $speed;
      $point = array(
        'time' => $subtime,
        'vindex' => $vindex,
        'major' => $major,
        'minor' => ++$minor,
        'lat' => $lat,
        'lng' => $lng,
        'angle' => $angle,
        'prev' => $prev
      );
      if ($speed >= 0) $point['speed'] = round($speed, 1);
      $path[] = $point;
    } else {# Invalid GPS data
      $lastindex = count($path) - 1; # index of the last path element
      if ($lastindex >= 0 && $path[$lastindex]['minor'] > 0) {# change it into a one-element section
        $path[$lastindex]['prev'] = $prev = $path[$lastindex]['minor'];
        $path[$lastindex]['major'] = ++$major;
        $path[$lastindex]['minor'] = $minor = 0;
      }
      if ($lastindex < 0 || array_key_exists('lng', $path[$lastindex])) {# Add an invalid point
        $major++;  $prev = 1;  $minor = 0;
        $lastlng = $lastlat = $seclat - $seclng = $lastangle = $sectionangle = 1234; # nonsense value
        $point = array(
          'time' => $subtime,
          'vindex' => $vindex,
          'major' => $major,
          'minor' => $minor,
          'prev' => $prev
        );
        $path[] = $point;
      }
    }
  }
}

# Remove the temporary files
if ($tmpfn) {
  unlink($subfn);
  unlink($tmpfn);
}

}# loop over files ======================================================================

# The last point must be a 1-element section
$N = count($path) - 1;
if ($N > 0 && $path[$N]['minor'] > 0) {
  $path[$N]['prev'] = $path[$N]['minor'];
  $major = $path[$N]['major'];
  $path[$N]['major'] = ++$major;
  $path[$N]['minor'] = 0;
  if (array_key_exists('lat', $path[$N]) && array_key_exists('lng', $path[$N])) {
    $path[$N]['idx'] = ++$idx;
  }
  $path[$N]['size'] = 1;
}

# Calculate section sizes
for ($n = $N - 1;  $n >= 0;  $n--) {
  $m = $path[$n]['major'];
  if ($m != $major) {
    $size = $path[$n]['minor'] + 1;
    $major = $m;
  }
  $path[$n]['size'] = $size;
}

# Last index
$LastIdx = count($path) - 1;

# JSON options
$opt = JSON_NUMERIC_CHECK | JSON_PRETTY_PRINT |  JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE;

# Definitions for the player frame
$tmpfn = tempnam(TMPDIR, '');
$fh = fopen($tmpfn, 'w');
fputs($fh, 'var playlist = ');
fputs($fh, json_encode($playlist, $opt) . ";\n");
fputs($fh, 'var offset = ');
fputs($fh, json_encode($offset, $opt) . ";\n");
fclose($fh);
$tmpfn = substr($tmpfn, strlen(TMPDIR)+1);
$duration = floor($tot);

?><!DOCTYPE html>
<html>
<head>
<script type="text/javascript">
var path = <?php echo json_encode($path, $opt); ?>;
var LastIdx = <?php echo $LastIdx; ?>;
</script>
</head>
<frameset cols="66%,34%">
  <frame name="player" src="player.php?js=<?php
         echo $tmpfn . '&' . "lang=$lang" . '&' . "duration=$duration";
  ?>" scrolling="no">
  <frameset rows="96%,4%">
    <frame name="map" src="map.php" scrolling="no">
    <frame name="gpsinfo" src="gpsinfo-<?php echo $lang ?>.html" scrolling="no">
  </frameset>
</frameset>
</html>
