// $Id: map.js 162 2014-07-04 22:24:30Z zw $

// Global variables
var PathElement, LastIdx = parent.LastIdx, VideoTime = 0, idx = -1, dist = 0;

// Map API variables
var map, gpsPath, marker;

// Control variables
var playing = false, fixnow = false, plist = false;

var CarWest = {
  anchor: new google.maps.Point({x: 32, y: 9}),
  size: new google.maps.Size({width: 64, height: 29}),
  url: 'carwest.png'
};

var CarEast = {
  anchor: new google.maps.Point({x: 32, y: 9}),
  size: new google.maps.Size({width: 64, height: 29}),
  url: 'careast.png'
};

var CarFront = {
  anchor: new google.maps.Point({x: 32, y: 9}),
  size: new google.maps.Size({width: 64, height: 45}),
  url: 'carfront.png'
};

var GPSlost = {
  anchor: new google.maps.Point({x: 32, y: 5}),
  size: new google.maps.Size({width: 64, height: 39}),
  url: 'gpslost.png'
};

var InitialIcon = 'gpslost.png';

// Find the initial position
if (parent.path !== undefined && LastIdx >= 0) {
  var E = parent.path[0];
  if ((E.lat === undefined || E.lng === undefined) && LastIdx >= E.size - E.minor) {
    E = parent.path[E.size - E.minor];
  }
  if (E.lat !== undefined) LAT = E.lat;
  if (E.lng !== undefined) LNG = E.lng;
  if (E.lat !== undefined && E.lng !== undefined) InitialIcon = 'carfront.png';
}

// The document in the GPS info frame
var GPSinfo = parent.gpsinfo.document;
var Player = parent.player;

// Fix the bounds
function FixBounds() {
  if ((!playing && !plist) || (Player.Pos !== undefined && Player.Pos > 0)) return;
  fixnow = plist = false;
  var bounds = map.getBounds();
  var NE = bounds.getNorthEast();
  var SW = bounds.getSouthWest();
  var pos = marker.getPosition();
  var norm = NE.lat() - SW.lat();
  var diffLat = (pos.lat() - SW.lat()) / norm * (NE.lat() - pos.lat()) / norm;
  norm = NE.lng() - SW.lng();
  var diffLng = (pos.lng() - SW.lng()) / norm * (NE.lng() - pos.lng()) / norm;
  if (diffLat < threshold || diffLng < threshold) map.panTo(pos);
}

// Test whether GPS data exist
function hasGPS(e) {
return e.lat !== undefined && e.lng !== undefined;
}

// Initialize the map
function initialize() {
  // Map
  var mapOptions = {
    center: new google.maps.LatLng(LAT, LNG),
    scrollwheel: false,
    zoom: 16
  };
  map = new google.maps.Map(document.getElementById("map-canvas"), mapOptions);
  // Marker (the head direction)
  marker = new google.maps.Marker({
    map: map,
    position: map.getCenter(),
    icon: InitialIcon,
    title: 'Current position'
  });
  google.maps.event.addListener(map, 'bounds_changed', FixBounds);
  gpsPath = new google.maps.MVCArray();
  var poly = new google.maps.Polyline({
    map: map,
    path: gpsPath,
    strokeColor: 'rgb(255, 0, 255)',
    strokeOpacity: 0.5,
    strokeWeight: 4
  });
  google.maps.event.addListener(poly, 'click', function (event) {
    var pos = event.latLng;
    var k, loc, elem, Delta = 12345, K = idx;
    for (k = 0;  k <= K;  k++) {
      var elem = parent.path[k];
      if (elem.lng === undefined || elem.lat === undefined) continue;
      var dlat = pos.lat() - elem.lat, dlng = pos.lng() - elem.lng;
      var delta = Math.sqrt(dlat * dlat + dlng * dlng);
      if (delta <= Delta) {
        loc = k;  Delta = delta;
      }
    }
    elem = parent.path[loc];
    Player.gotoTime(elem.time, elem.vindex);
  });
}
google.maps.event.addDomListener(window, 'load', initialize);

// Periodically watch the video time and update position
var delay = 333; // in miliseconds
function ZWtimer() {
  var vtm = VideoTime; // be sure our value does not change during function execution
  if (fixnow) FixBounds;
  // Bracket the path element based on the current value
  var L, U, M, step = 1;
  L = U = idx < 0 ? 0 : idx;
  while (U <= LastIdx && parent.path[U].time < vtm) {
    U += step;
    step += step;
  }
  while (L >= 0 && parent.path[L].time > vtm) {
    L -= step;
    step += step;
  }
  // Safeguarding
  if (L < 0) L = 0;
  if (U > LastIdx) U = LastIdx;
  // Search
  while (L <= U) {
    M = (L + U) >> 1;
    var tstime = parent.path[M].time;
    if (tstime <= vtm) L = M + 1;
    if (tstime >= vtm) U = M - 1;
  }
  if (L - U < 2) M = U;
  if (M < 0) M = 0; // in case subtitles do not start at the very beginning of the video
  if (M != idx) {
    var elem = parent.path[M]; // current path element
    if (PathElement === undefined) {// path is empty, insert the element if it contains GPS data
      if (hasGPS(elem)) gpsPath.push(new google.maps.LatLng(elem.lat, elem.lng));
    } else {
      if (PathElement.major != elem.major) {// change of section
        while (PathElement !== undefined && PathElement.minor > 0) {// remove section subpoints
          if (hasGPS(PathElement)) gpsPath.pop();
          PathElement = parent.path[--idx];
        }
        while (PathElement !== undefined && PathElement.major > elem.major) {// go back
          if (hasGPS(PathElement)) gpsPath.pop();
          idx -= PathElement.prev + PathElement.minor;
          PathElement = parent.path[idx];
        }
        while (PathElement === undefined || PathElement.major < elem.major) {// go forward
          if (PathElement === undefined) idx = 0;
          else idx += PathElement.size - PathElement.minor;
          var e = PathElement = parent.path[idx];
          if (hasGPS(e)) gpsPath.push(new google.maps.LatLng(e.lat, e.lng));
        }
      }
      if (PathElement !== undefined && PathElement.minor != elem.minor) {// update within section
        while (PathElement !== undefined && PathElement.minor > elem.minor) {// go back
          if (hasGPS(PathElement)) gpsPath.pop();
          PathElement = parent.path[--idx];
        }
        while (PathElement === undefined || PathElement.minor < elem.minor) {// go forward
          if (PathElement === undefined) idx = -1;
          var e = PathElement = parent.path[++idx];
          if (hasGPS(e)) gpsPath.push(new google.maps.LatLng(e.lat, e.lng));
        }
      }
      var k = gpsPath.getLength();
      if (k < 1 && PathElement !== undefined) {// handling the beginning of the path, is it needed?
        idx += PathElement.size - PathElement.minor;
        var e = PathElement = parent.path[idx];
        if (hasGPS(e)) gpsPath.push(new google.maps.LatLng(e.lat, e.lng));
      }
      k = gpsPath.getLength();
      if (k > 0) {// set the marker, calculate path length
      e = gpsPath.getAt(k-1);
        marker.setPosition({lat: e.lat(), lng: e.lng()});
        FixBounds();
        dist = google.maps.geometry.spherical.computeLength(gpsPath) / 1000;
        GPSinfo.getElementById('distance').innerHTML = dist.toFixed(2);
      } else marker.setIcon('gpslost.png');
    }
    var icon = 'carfront.png';
    if (elem.lat === undefined || elem.lng === undefined) icon = 'gpslost.png';
    else if (elem.speed === undefined || elem.speed > 1) {
      if (elem.angle < 180) icon = 'careast.png';
      else icon = 'carwest.png';
    }
    marker.setIcon(icon);
    PathElement = elem;
    idx = M;
    GPSinfo.getElementById('speed').innerHTML = PathElement.speed;
    GPSinfo.getElementById('heading').innerHTML = Math.round(PathElement.angle);
  }
  var avg = vtm == 0 ? 0 : dist / vtm * 3600;
  GPSinfo.getElementById('avg').innerHTML = avg.toFixed(1);
  setTimeout('ZWtimer()', delay);
}

// Do not start the timer if there is no path
if (LastIdx >= 0) setTimeout('ZWtimer()', delay);
